// Fill out your copyright notice in the Description page of Project Settings.


#include "ServerPluginActor.h"

// Sets default values
AServerPluginActor::AServerPluginActor()
{
 	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;

}

void AServerPluginActor::socketServerConnectionEventDelegate(const EServerSocketConnectionEventType type, const bool success, const FString message, const FString sessionID, const FString serverID)
{
	FColor messageColor = FColor::Red;
	if (success)
		messageColor = FColor::Green;
	if (GEngine)
		GEngine->AddOnScreenDebugMessage(-1, 10.0f, messageColor, serverID + " | " + sessionID + "| " + message);

}

void AServerPluginActor::serverReceiveTCPMessageEventDelegate(const FString sessionID, const FString message, const TArray<uint8>& byteArray, const FString serverID)
{
	if (GEngine)
		GEngine->AddOnScreenDebugMessage(-1, 2.0f, FColor::Cyan, serverID + " | " + sessionID + "| " + message);

	if (socketServerLib != nullptr) {
		socketServerLib->socketServerSendTCPMessageToClient(sessionID,  "I send the message back: "+message, TArray<uint8>(), true, serverID);
	}
}

// Called when the game starts or when spawned
void AServerPluginActor::BeginPlay()
{
	Super::BeginPlay();

	socketServerLib = USocketServerBPLibrary::getSocketServerTarget();
	socketServerLib->startTCPServer(serverIDGlobal, FString("0.0.0.0"),8888, EReceiveFilterServer::E_SAB);
	socketServerLib->onsocketServerConnectionEventDelegate.AddDynamic(this, &AServerPluginActor::socketServerConnectionEventDelegate);
	socketServerLib->onserverReceiveTCPMessageEventDelegate.AddDynamic(this, &AServerPluginActor::serverReceiveTCPMessageEventDelegate);

}


void AServerPluginActor::EndPlay(const EEndPlayReason::Type EndPlayReason)
{
	Super::EndPlay(EndPlayReason);
	USocketServerBPLibrary::getSocketServerTarget()->stopAllTCPServers();
	socketServerLib = nullptr;
}

// Called every frame
void AServerPluginActor::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

}

